from typing import Any

import numpy as np
from numpy.typing import _SupportsArray


class Index:
    def __index__(self) -> int:
        ...


a: "np.flatiter[np.ndarray]"
supports_array: _SupportsArray

a.base = Any  # E: Property "base" defined in "flatiter" is read-only
a.coords = Any  # E: Property "coords" defined in "flatiter" is read-only
a.index = Any  # E: Property "index" defined in "flatiter" is read-only
a.copy(order='C')  # E: Unexpected keyword argument

# NOTE: Contrary to `ndarray.__getitem__` its counterpart in `flatiter`
# does not accept objects with the `__array__` or `__index__` protocols;
# boolean indexing is just plain broken (gh-17175)
a[np.bool_()]  # E: No overload variant of "__getitem__"
a[Index()]  # E: No overload variant of "__getitem__"
a[supports_array]  # E: No overload variant of "__getitem__"



from typing import List

from numpy import (
    RankWarning as RankWarning,
    poly1d as poly1d,
)

__all__: List[str]

def poly(seq_of_zeros): ...
def roots(p): ...
def polyint(p, m=..., k=...): ...
def polyder(p, m=...): ...
def polyfit(x, y, deg, rcond=..., full=..., w=..., cov=...): ...
def polyval(p, x): ...
def polyadd(a1, a2): ...
def polysub(a1, a2): ...
def polymul(a1, a2): ...
def polydiv(u, v): ...



"""Tests for the optional typing-extensions dependency."""

import sys
import textwrap
import subprocess

CODE = textwrap.dedent(r"""
    import sys
    import importlib

    assert "typing_extensions" not in sys.modules
    assert "numpy.typing" not in sys.modules

    # Importing `typing_extensions` will now raise an `ImportError`
    sys.modules["typing_extensions"] = None
    assert importlib.import_module("numpy.typing")
""")


def test_no_typing_extensions() -> None:
    """Import `numpy.typing` in the absence of typing-extensions.

    Notes
    -----
    Ideally, we'd just run the normal typing tests in an environment where
    typing-extensions is not installed, but unfortunatelly this is currently
    impossible as it is an indirect hard dependency of pytest.

    """
    p = subprocess.run([sys.executable, '-c', CODE], capture_output=True)
    if p.returncode:
        raise AssertionError(
            f"Non-zero return code: {p.returncode!r}\n\n{p.stderr.decode()}"
        )




import sys
import pytest
from . import util

from numpy.testing import assert_equal, IS_PYPY

class TestBlockDocString(util.F2PyTest):
    code = """
      SUBROUTINE FOO()
      INTEGER BAR(2, 3)

      COMMON  /BLOCK/ BAR
      RETURN
      END
    """

    @pytest.mark.skipif(sys.platform=='win32',
                        reason='Fails with MinGW64 Gfortran (Issue #9673)')
    @pytest.mark.xfail(IS_PYPY,
                       reason="PyPy cannot modify tp_doc after PyType_Ready")
    def test_block_docstring(self):
        expected = "bar : 'i'-array(2,3)\n"
        assert_equal(self.module.block.__doc__, expected)



from typing import List

from numpy import (
    vectorize as vectorize,
)

from numpy.core.function_base import (
    add_newdoc as add_newdoc,
)

from numpy.core.multiarray import (
    add_docstring as add_docstring,
    bincount as bincount,
)
from numpy.core.umath import _add_newdoc_ufunc

__all__: List[str]

add_newdoc_ufunc = _add_newdoc_ufunc

def rot90(m, k=..., axes = ...): ...
def flip(m, axis=...): ...
def iterable(y): ...
def average(a, axis=..., weights=..., returned=...): ...
def asarray_chkfinite(a, dtype=..., order=...): ...
def piecewise(x, condlist, funclist, *args, **kw): ...
def select(condlist, choicelist, default=...): ...
def copy(a, order=..., subok=...): ...
def gradient(f, *varargs, axis=..., edge_order=...): ...
def diff(a, n=..., axis=..., prepend = ..., append = ...): ...
def interp(x, xp, fp, left=..., right=..., period=...): ...
def angle(z, deg=...): ...
def unwrap(p, discont = ..., axis=..., *, period=...): ...
def sort_complex(a): ...
def trim_zeros(filt, trim=...): ...
def extract(condition, arr): ...
def place(arr, mask, vals): ...
def disp(mesg, device=..., linefeed=...): ...
def cov(m, y=..., rowvar=..., bias=..., ddof=..., fweights=..., aweights=..., *, dtype=...): ...
def corrcoef(x, y=..., rowvar=..., bias = ..., ddof = ..., *, dtype=...): ...
def blackman(M): ...
def bartlett(M): ...
def hanning(M): ...
def hamming(M): ...
def i0(x): ...
def kaiser(M, beta): ...
def sinc(x): ...
def msort(a): ...
def median(a, axis=..., out=..., overwrite_input=..., keepdims=...): ...
def percentile(a, q, axis=..., out=..., overwrite_input=..., interpolation=..., keepdims=...): ...
def quantile(a, q, axis=..., out=..., overwrite_input=..., interpolation=..., keepdims=...): ...
def trapz(y, x=..., dx=..., axis=...): ...
def meshgrid(*xi, copy=..., sparse=..., indexing=...): ...
def delete(arr, obj, axis=...): ...
def insert(arr, obj, values, axis=...): ...
def append(arr, values, axis=...): ...
def digitize(x, bins, right=...): ...



from typing import List, Any
import numpy as np

AR_LIKE_b: List[bool]
AR_LIKE_u: List[np.uint32]
AR_LIKE_i: List[int]
AR_LIKE_f: List[float]
AR_LIKE_O: List[np.object_]

AR_U: np.ndarray[Any, np.dtype[np.str_]]

reveal_type(np.fix(AR_LIKE_b))  # E: numpy.ndarray[Any, numpy.dtype[numpy.floating[Any]]]
reveal_type(np.fix(AR_LIKE_u))  # E: numpy.ndarray[Any, numpy.dtype[numpy.floating[Any]]]
reveal_type(np.fix(AR_LIKE_i))  # E: numpy.ndarray[Any, numpy.dtype[numpy.floating[Any]]]
reveal_type(np.fix(AR_LIKE_f))  # E: numpy.ndarray[Any, numpy.dtype[numpy.floating[Any]]]
reveal_type(np.fix(AR_LIKE_O))  # E: Any
reveal_type(np.fix(AR_LIKE_f, out=AR_U))  # E: numpy.ndarray[Any, numpy.dtype[numpy.str_]]

reveal_type(np.isposinf(AR_LIKE_b))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isposinf(AR_LIKE_u))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isposinf(AR_LIKE_i))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isposinf(AR_LIKE_f))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isposinf(AR_LIKE_f, out=AR_U))  # E: numpy.ndarray[Any, numpy.dtype[numpy.str_]]

reveal_type(np.isneginf(AR_LIKE_b))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isneginf(AR_LIKE_u))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isneginf(AR_LIKE_i))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isneginf(AR_LIKE_f))  # E: numpy.ndarray[Any, numpy.dtype[numpy.bool_]]
reveal_type(np.isneginf(AR_LIKE_f, out=AR_U))  # E: numpy.ndarray[Any, numpy.dtype[numpy.str_]]



"""
Tests for :mod:`numpy.core.numeric`.

Does not include tests which fall under ``array_constructors``.

"""

from typing import List
import numpy as np

class SubClass(np.ndarray):
    ...

i8: np.int64

A: np.ndarray
B: List[int]
C: SubClass

reveal_type(np.count_nonzero(i8))  # E: int
reveal_type(np.count_nonzero(A))  # E: int
reveal_type(np.count_nonzero(B))  # E: int
reveal_type(np.count_nonzero(A, keepdims=True))  # E: Any
reveal_type(np.count_nonzero(A, axis=0))  # E: Any

reveal_type(np.isfortran(i8))  # E: bool
reveal_type(np.isfortran(A))  # E: bool

reveal_type(np.argwhere(i8))  # E: numpy.ndarray[Any, Any]
reveal_type(np.argwhere(A))  # E: numpy.ndarray[Any, Any]

reveal_type(np.flatnonzero(i8))  # E: numpy.ndarray[Any, Any]
reveal_type(np.flatnonzero(A))  # E: numpy.ndarray[Any, Any]

reveal_type(np.correlate(B, A, mode="valid"))  # E: numpy.ndarray[Any, Any]
reveal_type(np.correlate(A, A, mode="same"))  # E: numpy.ndarray[Any, Any]

reveal_type(np.convolve(B, A, mode="valid"))  # E: numpy.ndarray[Any, Any]
reveal_type(np.convolve(A, A, mode="same"))  # E: numpy.ndarray[Any, Any]

reveal_type(np.outer(i8, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.outer(B, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.outer(A, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.outer(A, A, out=C))  # E: SubClass

reveal_type(np.tensordot(B, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.tensordot(A, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.tensordot(A, A, axes=0))  # E: numpy.ndarray[Any, Any]
reveal_type(np.tensordot(A, A, axes=(0, 1)))  # E: numpy.ndarray[Any, Any]

reveal_type(np.isscalar(i8))  # E: bool
reveal_type(np.isscalar(A))  # E: bool
reveal_type(np.isscalar(B))  # E: bool

reveal_type(np.roll(A, 1))  # E: numpy.ndarray[Any, Any]
reveal_type(np.roll(A, (1, 2)))  # E: numpy.ndarray[Any, Any]
reveal_type(np.roll(B, 1))  # E: numpy.ndarray[Any, Any]

reveal_type(np.rollaxis(A, 0, 1))  # E: numpy.ndarray[Any, Any]

reveal_type(np.moveaxis(A, 0, 1))  # E: numpy.ndarray[Any, Any]
reveal_type(np.moveaxis(A, (0, 1), (1, 2)))  # E: numpy.ndarray[Any, Any]

reveal_type(np.cross(B, A))  # E: numpy.ndarray[Any, Any]
reveal_type(np.cross(A, A))  # E: numpy.ndarray[Any, Any]

reveal_type(np.indices([0, 1, 2]))  # E: numpy.ndarray[Any, Any]
reveal_type(np.indices([0, 1, 2], sparse=False))  # E: numpy.ndarray[Any, Any]
reveal_type(np.indices([0, 1, 2], sparse=True))  # E: tuple[numpy.ndarray[Any, Any]]

reveal_type(np.binary_repr(1))  # E: str

reveal_type(np.base_repr(1))  # E: str

reveal_type(np.allclose(i8, A))  # E: bool
reveal_type(np.allclose(B, A))  # E: bool
reveal_type(np.allclose(A, A))  # E: bool

reveal_type(np.isclose(i8, A))  # E: Any
reveal_type(np.isclose(B, A))  # E: Any
reveal_type(np.isclose(A, A))  # E: Any

reveal_type(np.array_equal(i8, A))  # E: bool
reveal_type(np.array_equal(B, A))  # E: bool
reveal_type(np.array_equal(A, A))  # E: bool

reveal_type(np.array_equiv(i8, A))  # E: bool
reveal_type(np.array_equiv(B, A))  # E: bool
reveal_type(np.array_equiv(A, A))  # E: bool



import os
import sys
from distutils.command.build import build as old_build
from distutils.util import get_platform
from numpy.distutils.command.config_compiler import show_fortran_compilers

class build(old_build):

    sub_commands = [('config_cc',     lambda *args: True),
                    ('config_fc',     lambda *args: True),
                    ('build_src',     old_build.has_ext_modules),
                    ] + old_build.sub_commands

    user_options = old_build.user_options + [
        ('fcompiler=', None,
         "specify the Fortran compiler type"),
        ('warn-error', None,
         "turn all warnings into errors (-Werror)"),
        ('cpu-baseline=', None,
         "specify a list of enabled baseline CPU optimizations"),
        ('cpu-dispatch=', None,
         "specify a list of dispatched CPU optimizations"),
        ('disable-optimization', None,
         "disable CPU optimized code(dispatch,simd,fast...)"),
        ('simd-test=', None,
         "specify a list of CPU optimizations to be tested against NumPy SIMD interface"),
        ]

    help_options = old_build.help_options + [
        ('help-fcompiler', None, "list available Fortran compilers",
         show_fortran_compilers),
        ]

    def initialize_options(self):
        old_build.initialize_options(self)
        self.fcompiler = None
        self.warn_error = False
        self.cpu_baseline = "min"
        self.cpu_dispatch = "max -xop -fma4" # drop AMD legacy features by default
        self.disable_optimization = False
        """
        the '_simd' module is a very large. Adding more dispatched features
        will increase binary size and compile time. By default we minimize
        the targeted features to those most commonly used by the NumPy SIMD interface(NPYV),
        NOTE: any specified features will be ignored if they're:
            - part of the baseline(--cpu-baseline)
            - not part of dispatch-able features(--cpu-dispatch)
            - not supported by compiler or platform
        """
        self.simd_test = "BASELINE SSE2 SSE42 XOP FMA4 (FMA3 AVX2) AVX512F AVX512_SKX VSX VSX2 VSX3 NEON ASIMD"

    def finalize_options(self):
        build_scripts = self.build_scripts
        old_build.finalize_options(self)
        plat_specifier = ".{}-{}.{}".format(get_platform(), *sys.version_info[:2])
        if build_scripts is None:
            self.build_scripts = os.path.join(self.build_base,
                                              'scripts' + plat_specifier)

    def run(self):
        old_build.run(self)



"""Test the runtime usage of `numpy.typing`."""

from __future__ import annotations

import sys
from typing import get_type_hints, Union, Tuple, NamedTuple

import pytest
import numpy as np
import numpy.typing as npt

try:
    from typing_extensions import get_args, get_origin
    SKIP = False
except ImportError:
    SKIP = True


class TypeTup(NamedTuple):
    typ: type
    args: Tuple[type, ...]
    origin: None | type


if sys.version_info >= (3, 9):
    NDArrayTup = TypeTup(npt.NDArray, npt.NDArray.__args__, np.ndarray)
else:
    NDArrayTup = TypeTup(npt.NDArray, (), None)

TYPES = {
    "ArrayLike": TypeTup(npt.ArrayLike, npt.ArrayLike.__args__, Union),
    "DTypeLike": TypeTup(npt.DTypeLike, npt.DTypeLike.__args__, Union),
    "NBitBase": TypeTup(npt.NBitBase, (), None),
    "NDArray": NDArrayTup,
}


@pytest.mark.parametrize("name,tup", TYPES.items(), ids=TYPES.keys())
@pytest.mark.skipif(SKIP, reason="requires typing-extensions")
def test_get_args(name: type, tup: TypeTup) -> None:
    """Test `typing.get_args`."""
    typ, ref = tup.typ, tup.args
    out = get_args(typ)
    assert out == ref


@pytest.mark.parametrize("name,tup", TYPES.items(), ids=TYPES.keys())
@pytest.mark.skipif(SKIP, reason="requires typing-extensions")
def test_get_origin(name: type, tup: TypeTup) -> None:
    """Test `typing.get_origin`."""
    typ, ref = tup.typ, tup.origin
    out = get_origin(typ)
    assert out == ref


@pytest.mark.parametrize("name,tup", TYPES.items(), ids=TYPES.keys())
def test_get_type_hints(name: type, tup: TypeTup) -> None:
    """Test `typing.get_type_hints`."""
    typ = tup.typ

    # Explicitly set `__annotations__` in order to circumvent the
    # stringification performed by `from __future__ import annotations`
    def func(a): pass
    func.__annotations__ = {"a": typ, "return": None}

    out = get_type_hints(func)
    ref = {"a": typ, "return": type(None)}
    assert out == ref


@pytest.mark.parametrize("name,tup", TYPES.items(), ids=TYPES.keys())
def test_get_type_hints_str(name: type, tup: TypeTup) -> None:
    """Test `typing.get_type_hints` with string-representation of types."""
    typ_str, typ = f"npt.{name}", tup.typ

    # Explicitly set `__annotations__` in order to circumvent the
    # stringification performed by `from __future__ import annotations`
    def func(a): pass
    func.__annotations__ = {"a": typ_str, "return": None}

    out = get_type_hints(func)
    ref = {"a": typ, "return": type(None)}
    assert out == ref


def test_keys() -> None:
    """Test that ``TYPES.keys()`` and ``numpy.typing.__all__`` are synced."""
    keys = TYPES.keys()
    ref = set(npt.__all__)
    assert keys == ref



import pytest
from numpy.core._simd import targets
"""
This testing unit only for checking the sanity of common functionality,
therefore all we need is just to take one submodule that represents any
of enabled SIMD extensions to run the test on it and the second submodule
required to run only one check related to the possibility of mixing
the data types among each submodule.
"""
npyvs = [npyv_mod for npyv_mod in targets.values() if npyv_mod and npyv_mod.simd]
npyv, npyv2 = (npyvs + [None, None])[:2]

unsigned_sfx = ["u8", "u16", "u32", "u64"]
signed_sfx = ["s8", "s16", "s32", "s64"]
fp_sfx = ["f32"]
if npyv and npyv.simd_f64:
    fp_sfx.append("f64")

int_sfx = unsigned_sfx + signed_sfx
all_sfx = unsigned_sfx + int_sfx

@pytest.mark.skipif(not npyv, reason="could not find any SIMD extension with NPYV support")
class Test_SIMD_MODULE:

    @pytest.mark.parametrize('sfx', all_sfx)
    def test_num_lanes(self, sfx):
        nlanes = getattr(npyv, "nlanes_" + sfx)
        vector = getattr(npyv, "setall_" + sfx)(1)
        assert len(vector) == nlanes

    @pytest.mark.parametrize('sfx', all_sfx)
    def test_type_name(self, sfx):
        vector = getattr(npyv, "setall_" + sfx)(1)
        assert vector.__name__ == "npyv_" + sfx

    def test_raises(self):
        a, b = [npyv.setall_u32(1)]*2
        for sfx in all_sfx:
            vcb = lambda intrin: getattr(npyv, f"{intrin}_{sfx}")
            pytest.raises(TypeError, vcb("add"), a)
            pytest.raises(TypeError, vcb("add"), a, b, a)
            pytest.raises(TypeError, vcb("setall"))
            pytest.raises(TypeError, vcb("setall"), [1])
            pytest.raises(TypeError, vcb("load"), 1)
            pytest.raises(ValueError, vcb("load"), [1])
            pytest.raises(ValueError, vcb("store"), [1], getattr(npyv, f"reinterpret_{sfx}_u32")(a))

    @pytest.mark.skipif(not npyv2, reason=(
        "could not find a second SIMD extension with NPYV support"
    ))
    def test_nomix(self):
        # mix among submodules isn't allowed
        a = npyv.setall_u32(1)
        a2 = npyv2.setall_u32(1)
        pytest.raises(TypeError, npyv.add_u32, a2, a2)
        pytest.raises(TypeError, npyv2.add_u32, a, a)

    @pytest.mark.parametrize('sfx', unsigned_sfx)
    def test_unsigned_overflow(self, sfx):
        nlanes = getattr(npyv, "nlanes_" + sfx)
        maxu = (1 << int(sfx[1:])) - 1
        maxu_72 = (1 << 72) - 1
        lane = getattr(npyv, "setall_" + sfx)(maxu_72)[0]
        assert lane == maxu
        lanes = getattr(npyv, "load_" + sfx)([maxu_72] * nlanes)
        assert lanes == [maxu] * nlanes
        lane = getattr(npyv, "setall_" + sfx)(-1)[0]
        assert lane == maxu
        lanes = getattr(npyv, "load_" + sfx)([-1] * nlanes)
        assert lanes == [maxu] * nlanes

    @pytest.mark.parametrize('sfx', signed_sfx)
    def test_signed_overflow(self, sfx):
        nlanes = getattr(npyv, "nlanes_" + sfx)
        maxs_72 = (1 << 71) - 1
        lane = getattr(npyv, "setall_" + sfx)(maxs_72)[0]
        assert lane == -1
        lanes = getattr(npyv, "load_" + sfx)([maxs_72] * nlanes)
        assert lanes == [-1] * nlanes
        mins_72 = -1 << 71
        lane = getattr(npyv, "setall_" + sfx)(mins_72)[0]
        assert lane == 0
        lanes = getattr(npyv, "load_" + sfx)([mins_72] * nlanes)
        assert lanes == [0] * nlanes

    def test_truncate_f32(self):
        f32 = npyv.setall_f32(0.1)[0]
        assert f32 != 0.1
        assert round(f32, 1) == 0.1

    def test_compare(self):
        data_range = range(0, npyv.nlanes_u32)
        vdata = npyv.load_u32(data_range)
        assert vdata == list(data_range)
        assert vdata == tuple(data_range)
        for i in data_range:
            assert vdata[i] == data_range[i]



from typing import List

from numpy import (
    format_parser as format_parser,
    record as record,
    recarray as recarray,
)

__all__: List[str]

def fromarrays(
    arrayList,
    dtype=...,
    shape=...,
    formats=...,
    names=...,
    titles=...,
    aligned=...,
    byteorder=...,
): ...
def fromrecords(
    recList,
    dtype=...,
    shape=...,
    formats=...,
    names=...,
    titles=...,
    aligned=...,
    byteorder=...,
): ...
def fromstring(
    datastring,
    dtype=...,
    shape=...,
    offset=...,
    formats=...,
    names=...,
    titles=...,
    aligned=...,
    byteorder=...,
): ...
def fromfile(
    fd,
    dtype=...,
    shape=...,
    offset=...,
    formats=...,
    names=...,
    titles=...,
    aligned=...,
    byteorder=...,
): ...
def array(
    obj,
    dtype=...,
    shape=...,
    offset=...,
    strides=...,
    formats=...,
    names=...,
    titles=...,
    aligned=...,
    byteorder=...,
    copy=...,
): ...
